#!/usr/bin/env python

from __future__ import with_statement
import Live
import time
from _Framework.ButtonElement import ButtonElement
from _Framework.ButtonMatrixElement import ButtonMatrixElement
from _Framework.ChannelStripComponent import ChannelStripComponent
from _Framework.ClipSlotComponent import ClipSlotComponent
from _Framework.CompoundComponent import CompoundComponent
from _Framework.ControlElement import ControlElement
from _Framework.ControlSurface import ControlSurface
from _Framework.ControlSurfaceComponent import ControlSurfaceComponent 
from _Framework.InputControlElement import *
from _Framework.MixerComponent import MixerComponent
from _Framework.SceneComponent import SceneComponent
from _Framework.SessionComponent import SessionComponent
from _Framework.TransportComponent import TransportComponent
from ConfigurableButtonElement import ConfigurableButtonElement
from MIDI_Map import *

class DW(ControlSurface):
    __module__ = __name__
    __doc__ = " Digital Warrior Control Script "
    
    def __init__(self, c_instance):
        ControlSurface.__init__(self, c_instance)
        with self.component_guard():
            self._suppress_session_highlight = True
            self._suppress_send_midi = True
            self.log_message("Digital Warrior Control Script Started!")
            self.box_width = box_width
            self.box_height = box_height
            self._session = None
            self._mixer = None
            self._setup_mixer_control()
            self._setup_session_control()
            self.session.set_offsets(0,0)# first script start from 0
            self.session.set_mixer(self._mixer)
            self._suppress_session_highlight = False

    def _setup_session_control(self):
        is_momentary = True
        num_tracks = box_width
        num_scenes = box_height
        self.session = SessionComponent(num_tracks, num_scenes)
        self.session.name = 'Session_Control'
        up_button = ButtonElement(is_momentary, MIDI_NOTE_TYPE, CHANNEL, up) 
        up_button.name = 'Bank_Select_Up_Button'
        down_button = ButtonElement(is_momentary, MIDI_NOTE_TYPE, CHANNEL, down)
        down_button.name = 'Bank_Select_Down_Button'
        self.session.set_scene_bank_buttons(down_button, up_button)
        right_button = ButtonElement(is_momentary, MIDI_NOTE_TYPE, CHANNEL, right)
        right_button.name = 'Bank_Select_Right_Button'
        left_button = ButtonElement(is_momentary, MIDI_NOTE_TYPE, CHANNEL, left)
        left_button.name = 'Bank_Select_Left_Button'
        self.session.set_track_bank_buttons(right_button, left_button)
        matrix = ButtonMatrixElement()
        matrix.name = 'Button_Matrix'
        # launch box
        #launch_button_list.reverse()
        self.clip_loaded_stopped = WHITE
        self.clip_currently_playing = GREEN
        self.clip_triggered_to_play = YELLOW
        self.clip_recording_value = RED
        for scene_index in range(self.box_height):
            scene = self.session.scene(scene_index)
            scene.name = 'Scene_' + str(scene_index)
            button_row = []
            for track_index in range(self.box_width): 
                button = ButtonElement(is_momentary, MIDI_NOTE_TYPE, CHANNEL, launch_button_list[scene_index][track_index])
                button.name = str(track_index) + '_Clip_' + str(scene_index) + '_Button'
                button_row.append(button)
                clip_slot = scene.clip_slot(track_index)
                clip_slot.name = str(track_index) + '_Clip_Slot_' + str(scene_index)
                clip_slot.set_stopped_value(self.clip_loaded_stopped )
                clip_slot.set_started_value(self.clip_currently_playing)
                clip_slot.set_recording_value(self.clip_recording_value)
                clip_slot.set_triggered_to_play_value(self.clip_triggered_to_play)  
                clip_slot.set_launch_button(button)
            matrix.add_row(tuple(button_row))
        self.set_highlighting_session_component(self.session)
        return None

    def _setup_mixer_control(self):
        is_momentary = True
        self._mixer = MixerComponent(4)
        self._mixer.name = 'Mixer'
        self._mixer.selected_strip().name = 'Selected_Channel_Strip'  
        for index in range(self.box_width):
            strip = self._mixer.channel_strip(index)
            strip.name = 'Channel_Strip_' + str(index)
            strip.set_solo_button(self.set_button(CHANNEL, solo_track_buttons[index], CYAN, OFF)) 
            strip.set_arm_button(self.set_button(CHANNEL, arm_track_buttons[index], RED, OFF))
            strip.set_mute_button(self.set_button(CHANNEL, mute_track_buttons[index], YELLOW, OFF))
            strip.set_select_button(self.set_button(CHANNEL, select_track_buttons[index], GREEN, OFF))
            strip.set_invert_mute_feedback(True)

    def set_button(self, _channel, _note, _on_color, _off_color):
        button = None
        if not _note is -1:
            button = ConfigurableButtonElement(True, MIDI_NOTE_TYPE, _channel, _note, _on_color, _off_color)
        return button

    def disconnect(self):
        ControlSurface.disconnect(self)
        return None
